/*
 * Created on 20.02.2015
 *
 * Author:      Eric
 * Copyright:   Contecon Software GmbH
 * 
 * History:     20.02.2015 created
 * 
 */
package de.contecon.picapport.plugin.dcraw;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.List;

import de.contecon.picapport.plugins.IPicApportPlugInLogger;

public class DcRawProcessWrapper {

    private IPicApportPlugInLogger logger;
    private StringWriter swStdOut;
    private StringWriter swStdErr;
    private int status;
    
    public DcRawProcessWrapper(IPicApportPlugInLogger logger, File directory, List command) throws Exception {
      this(logger, directory, command, null);
    }
 
    public DcRawProcessWrapper(IPicApportPlugInLogger logger, File directory, List command, File redirectStdOut) throws Exception {
        this.logger=logger;
        swStdOut = new StringWriter();
        swStdErr = new StringWriter();
        
        logger.LogDebugMessage("DcRawPlugin.execute: "+command);
        
        ProcessBuilder pb = new ProcessBuilder(command);
        if(directory != null) {
            pb.directory(directory);
        }
        Process process = pb.start();
        Thread seInfo = null;
        if(redirectStdOut != null) {
          seInfo = new StreamHandlerBinaryFile(process.getInputStream(), redirectStdOut);
          } else {
          seInfo = new StreamHandlerString(process.getInputStream(), new PrintWriter(swStdOut, true));
          }
        Thread seError = new StreamHandlerString(process.getErrorStream(), new PrintWriter(swStdErr, true));
        seInfo.start();
        seError.start();
        status = process.waitFor();
        seInfo.join();
        seError.join();
    }
 
    public String getErrors() {
        return swStdErr.toString();
    }
 
    public String getSwStdOut() {
        return swStdOut.toString();
    }
 
    public int getStatus() {
        return status;
    }
 
    class StreamHandlerString extends Thread {
        private InputStream in;
        private PrintWriter pw;
 
        StreamHandlerString(InputStream in, PrintWriter pw) {
            this.in = in;
            this.pw = pw;
        }
 
        @Override
        public void run() {
            try(BufferedReader br = new BufferedReader(new InputStreamReader(in))) {
                String line = null;
                while ( (line = br.readLine()) != null) {
                    pw.println(line);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
  
    class StreamHandlerBinaryFile extends Thread {
      private InputStream ins;
      private File outFile;
      
      StreamHandlerBinaryFile(InputStream in, File outputFile) {
          this.ins = in;
          this.outFile = outputFile;
      }

      @Override
      public void run() {
        outFile.delete();
        try (BufferedInputStream in = new BufferedInputStream(this.ins, 100000);
             BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(outFile), 100000)) {
          int byteRead;
          while ((byteRead = in.read()) != -1) {
             out.write(byteRead);
          }

       } catch (IOException ex) {
          ex.printStackTrace();
       }

      }
  }
  
}
